package org.jtheque.utils.bean;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jtheque.utils.StringUtils;
import org.jtheque.utils.Constants;

import java.util.StringTokenizer;
import java.util.Map;
import java.util.LinkedHashMap;

/**
 * A version of a module or an other part of the application.
 *
 * @author Baptiste Wicht
 */
public final class Version implements Comparable<Version> {
    private final String strVersion;

    private int hash;

    private static final Map<String, Integer> CODES;
    
    private static final int HIGHEST_CODE = 10;

    static {
        CODES = new LinkedHashMap<String, Integer>(11);

        CODES.put("beta", 2);
        CODES.put("b", 2);
        CODES.put("B", 2);

        CODES.put("alpha", 1);
        CODES.put("a", 1);
        CODES.put("A", 1);

        CODES.put("milestone", 3);
        CODES.put("m", 3);
        CODES.put("M", 3);

        CODES.put("rc", 4);
        CODES.put("RC", 4);
    }

    /**
     * Construct a new JThequeVersion with the String version.
     *
     * @param version The version
     */
    public Version(String version) {
        super();

        strVersion = version;
    }

    /**
     * Indicate if the version is greather than an other version.
     *
     * @param otherVersion The version we want to compare to
     * @return true if the current version is greather than the other else false
     */
    public boolean isGreaterThan(Version otherVersion) {
        if (strVersion.compareTo(otherVersion.strVersion) == 0) {
            return false;
        }

        StringTokenizer currentTokens = new StringTokenizer(strVersion, ".");
        StringTokenizer otherTokens = new StringTokenizer(otherVersion.strVersion, ".");

        while (currentTokens.hasMoreTokens()) {
            if (otherTokens.hasMoreTokens()) {
                String current = currentTokens.nextToken();
                String other = otherTokens.nextToken();

                String codeStrCurrent = extractCode(current);
                String codeStrOther = extractCode(other);

                if(StringUtils.isEmpty(codeStrCurrent) && StringUtils.isEmpty(codeStrOther)){
                    int compare = Integer.valueOf(current).compareTo(Integer.valueOf(other));

                    if (compare != 0) {
                        return compare > 0;
                    }
                } else {
                    return compareCode(current, other, codeStrCurrent, codeStrOther);
                }
            } else {
                return StringUtils.isEmpty(extractCode(currentTokens.nextToken()));
            }
        }

        return true;
    }

    /**
     * Compare the codes of two version.
     *
     * @param current The current version.
     * @param other The other version.
     * @param codeStrCurrent The code of the current version.
     * @param codeStrOther The code of the other version.
     *
     * @return true if the current code is most valued than the other.
     */
    private static boolean compareCode(String current, String other, String codeStrCurrent, String codeStrOther) {
        int codeCurrent = intCode(codeStrCurrent);
        int codeOther = intCode(codeStrOther);

        if(codeCurrent == codeOther){
            String strCurrent = current.replace(codeStrCurrent, StringUtils.EMPTY_STRING);
            String strOther = other.replace(codeStrOther, StringUtils.EMPTY_STRING);

            return Integer.valueOf(strCurrent).compareTo(Integer.valueOf(strOther)) >= 0;
        } else {
            return codeCurrent > codeOther;
        }
    }

    /**
     * Return the int code of the string code.
     *
     * @param strCode The string code.
     *
     * @return The int value of the string code.
     */
    private static int intCode(String strCode) {
        return StringUtils.isEmpty(strCode) ? HIGHEST_CODE : CODES.get(strCode);
    }

    /**
     * Extract the code contained in the version.
     *
     * @param version The version string.
     *
     * @return The code contained in the version or an empty String if there is no code in this version. 
     */
    private static String extractCode(String version) {
        for(Map.Entry<String, Integer> entry : CODES.entrySet()){
            if(version.contains(entry.getKey())){
                return entry.getKey();
            }
        }

        return "";
    }

    /**
     * Returns the string version.
     *
     * @return The version
     */
    public String getVersion() {
        return strVersion;
    }

    @Override
    public String toString() {
        return strVersion;
    }

    @Override
    public int hashCode() {
        if (hash == 0) {
            hash = Constants.HASH_CODE_START;

            if (strVersion != null) {
                hash = Constants.HASH_CODE_PRIME * strVersion.hashCode() + hash;
            }
        }

        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (obj != null && obj.getClass() == getClass()) {
            Version version = (Version) obj;
            if (compareTo(version) == 0) {
                return true;
            }
        }

        return false;
    }

    @Override
    public int compareTo(Version o) {
        if (strVersion.compareTo(o.strVersion) == 0) {
            return 0;
        }

        if (isGreaterThan(o)){
			return 1;
		}
		
		return -1;
	}
}